<?php
/*************************************************************************************
   Copyright notice

   (c) 2002-2008 Oliver Georgi (oliver@phpwcms.de) // All rights reserved.

   This script is part of PHPWCMS. The PHPWCMS web content management system is
   free software; you can redistribute it and/or modify it under the terms of
   the GNU General Public License as published by the Free Software Foundation;
   either version 2 of the License, or (at your option) any later version.

   The GNU General Public License can be found at http://www.gnu.org/copyleft/gpl.html
   A copy is found in the textfile GPL.txt and important notices to the license
   from the author is found in LICENSE.txt distributed with these scripts.

   This script is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
   PARTICULAR PURPOSE.  See the GNU General Public License for more details.

   This copyright notice MUST APPEAR in all copies of the script!
*************************************************************************************/
/*
   V1.0   - 19.10.08 Version for {GALLERYX:...} V1.0    21.10.08 
   V1.1   - 26.11.08 Version for {GALLERYX:...} V1.1    26.11.08 
   V1.2   - 11.12.08 Version for {GALLERYX:...} V1.2    11.12.08 
   V1.2.1 - 11.01.09 Version for {GALLERYX:...} V1.2.1  11.01.09 
   V1.2.2 - 21.02.09 Version for {GALLERYX:...} V1.2.2  21.02.09 
   patched by K.Heermann (flip-flop) http://planmatrix.de and RCS / robin c. sherwood 

*************************************************************************************/
/**
 * phpwcmsImageGallery
 *
 * A file center driven gallery for phpwcms
 *
 */

class phpwcmsImageGalleryX {

	var $url								= '';
	var $alias							= '';
	var $images							= array();
	var $query_separator		= '/';
	var $download_on				= 0; 				// [0=off|1=download|3=download & direct]
// -kh	var $download						= false;
// -kh	var $download_direct		= false;

	// string defines the sorting mode of sub sub gallery images
	// ASC/DESC = get images sorted by date 
	// RAND = get images by random)
	// +kh NAME-ASC, NAME-DESC
	// SORT-ASC, SORT-DESC, SORT-NAME-ASC, SORT-NAME-DESC, SORT-CREATE-ASC, SORT-CREATE-DESC  (since r304)
	// NAME-ASC, NAME-DESC, CREATE-ASC, CREATE-DESC
	var $image_sort					= 'DESC';
	
	var $image_src					= '';

	// integer the limit of how many images should be retrieved
	var $image_limit				= 0;
	
	var $thumb_width				= 200;
	var $thumb_height				= 150;
	// ---------- +kh
	var $thumb_count_to_min	= 20;
	var $thumb_width_min		= 200;
	var $thumb_height_min		= 150;
	// ----------
	var $width							= 600;
	var $height							= 400;
	var $zoom								= true;
	var $lightbox						= true;
	var $lightbox_linebreak = true; // capture: [0|1] [0=off (old behavior) | 1=on (no [BR] needed for linebreak)]

	var $galleries					= array();
	var $gallery						= array();
	
	// define what kind of file storage folders should be taken
	// true = only folders marked as gallery root or sub directory
	var $gallery_only				= true;
	
	// define the root ID of gallery = where to start recursive search
	// by default root ID = 0 which is top level
	var $gallery_rootid			= 0;
	
	// define the sub_sub gallery id = default 0
	var $gallery_sub_subid	= 0;
	
	// integer the max depth of recursive folder search, THREAD mode only 
	var $depth							= 0;
	
	// string defines the sorting mode of galleries:
	// SORT-ASC, SORT-DESC, SORT-NAME-ASC, SORT-NAME-DESC, SORT-CREATE-ASC, SORT-CREATE-DESC  (since r304)
	// NAME-ASC, NAME-DESC, CREATE-ASC, CREATE-DESC, SPECIAL-IMG
	// get images sorted by name or create date
	var $gallery_sort	= 'NAME-DESC';
	var $_sort							= '';
	
	// string the search mode - FLAT/THREAD (FLAT = default)
	var $gallery_mode				= 'FLAT'; //'THREAD'; // 'FLAT';
	
	// gallery name can be prefixed by date YYYY-MM-DD,
	// use sort by NAME-ASC or NAME-DESC,
	// this date will be used as gallery date then and
	// is stripped from name if $use_name_date = true
	var $use_name_date			= true;
	
	// [0|1|2|3] off | title=on | date=on | title and date = on
	var $list_title_date					= 3; // +kh	
	
	// [0|1|2|3] [0=off | 1=info |2=off | 3=info and date ]   in sub gallery (list)
	var $list_descr_info_date_on	= 3; // no separate date available
	
	
	// template definitions
	var $list_header				= '	<div class="galleries">';
	var $list_footer				= '</div>';
	var $list_template			= '[TAB]<div class="gallery">[LF][TAB][TAB]{TITLE} [LF][TAB][TAB]{THUMBNAIL} [LF][TAB][TAB] {DESCRIPTION}[LF][TAB]{COPYRIGHT}[LF][TAB]</div>';
	var $list_title_prefix	= '<h3>[LINK]';
	var $list_title_suffix	= '[/LINK]</h3>';
	var $list_descr_prefix	= '<div><strong>{DATE:m/d/y lang=EN}</strong> &#8211; ';
	var $list_descr_suffix	= '</div>';
	
// string defines the sorting mode of the list Sub Gallery thumbs:
	// RAND = get images by random)
	// SORT-ASC, SORT-DESC, SORT-NAME-ASC, SORT-NAME-DESC, SORT-CREATE-ASC, SORT-CREATE-DESC  (since r304)
	// NAME-ASC, NAME-DESC, CREATE-ASC, CREATE-DESC, 
	// LIST-SPECIAL-THUMB -> A special thumb for gallery listing with sort number = -1, -2, -3, ....
	var $list_thumbnail_sort			= '';
	
	// define if an list gallery thumbnail should be shown
	// 0 = show none, 1 show first, 2 show first and second ...
	var $list_thumbnail						= 1;
	var $list_thumbnail_caption 	= false; // +kh
	var $list_thumbnail_crop			= 0;
	var $list_thumbnail_width			= 150;
	var $list_thumbnail_height		= 150;
	var $list_thumbnail_prefix		= '<div class="thumbnail">';
	var $list_thumbnail_suffix		= '</div>';
	
	// [0|1|2|3] off | title=on | date=on | title and date = on
	var $detail_title_date_on		= 3; // +kh
	var $detail_header					= '<div class="gallery">';
	var $detail_footer					= '</div>';
	var $detail_title_prefix		= '<h2><strong>{DATE:m/d/y lang=EN}</strong> &#8211; ';
	var $detail_title_suffix		= '</h2>';
	var $detail_descr_prefix		= '<div>';
	var $detail_descr_suffix		= '</div>';
	var $detail_thumbnail_crop		= 0;
	var $detail_thumbnail_prefix	= '<div class="thumbnail">';
	var $detail_thumbnail_suffix	= '{DOWNLOAD}</div>';
	
	// [0|1|2|3] [0=off | 1=capt |2=light | 3=capt&light ]   in sub sub gallery
	var $detail_caption_on				= 3;
	var $detail_caption_text			= '';
	var $detail_caption_title			= '[[[[[]]]]]'; // please no blank, important for right caption carry to lightbox
	var $detail_caption_alt				= '';
	var $detail_descr_info				= true;  // +kh: [0|1] Description from list in detail off/on
	var $detail_zoom_crop					= false;
	var $detail_caption_prefix		= '<div class="caption">';
	var $detail_caption_suffix		= '</div>';
	
	
// copyright ----------- +kh	
/* 
[0|1|2|3|4|5|6|7] (int) in sub sub gallery
[0=off|1=cap|2=light|3=capt&light|4=down|5=down&capt|6=down&light|7=down&light&capt]
 0 = copyright           = off           (00000000)
 1 = caption             = on            (00000001)
 2 = lightbox            = on            (00000010)
 3 = caption & lightbox  = on            (00000011)
 4 = download            = on            (00000100)
 5 = download & caption  = on            (00000101)
 6 = download & lightbox = on            (00000110)
 7 = download & lightbox & caption = on  (00000111)
*/
	var $detail_copyr_on						= 7;
	
	var $detail_copyr_prefix				= '';
	var $detail_copyr_suffix				= '';
	var $detail_copyr_text					= '';  // set the fallback copyright text
	var $detail_copyr_text_prefix		= ''; // 
// -----------
	var $detail_gallery_back			= '<div class="backlink">[LINK]Show all galleries[/LINK]</div>';
	
// gallery images ==========
/*
	var $item_template			= '[TAB]{TITLE} [LF][TAB][TAB]{THUMBNAIL} [LF][TAB][TAB] {DESCRIPTION} [LF][TAB] {COPYRIGHT}';
	var $item_header				= '<div class="gallery">';
	var $item_footer				= '</div>';
	var $item_title_prefix	= '<h3>[LINK]';
	var $item_title_suffix	= '[/LINK]</h3>';
	var $item_descr_prefix	= '<div><strong>{DATE:m/d/y lang=EN}</strong> &#8211; ';
	var $item_descr_suffix	= '</div>';
*/
	var $item_download_template		= 'default';

/*
	// define if an gallery thumbnail should be shown
	// 0 = show none, 1 show first ...
	var $item_thumbnail					= 1;
	var $item_thumbnail_width		= 150;
	var $item_thumbnail_height	= 150;
	var $item_thumbnail_crop		= 0;
	var $item_thumbnail_prefix	= '<div class="thumbnail">';
	var $item_thumbnail_suffix	= '</div>';
*/	
// -------- + t
	var $list_image_header			= '<div class="gallery-image-list">';
	var $list_image_footer			= '</div>';
	var $list_image_prefix			= '<div class="image">[LINK]';
	var $list_image_suffix			= '[/LINK]</div>';
	var $list_image_width				= 100;
	var $list_image_height			= 100;
	var $list_image_crop				= 0;
// --------
	var $line_break							= "\n\t";
	
// -------------------------- +kh only for developement
var $mydump1 = false; 
// if ($this->mydump1) {echo 'text: '; dumpVar($my_var);}
//  -------------------------- */
	
	/**
	 * Initialize some defaults
	 */	
	 
	function phpwcmsImageGalleryX() {
	
		$this->_getGallerySortString();
	
		if($this->alias === '') {
			global $aktion;
			$this->alias = defined('PHPWCMS_ALIAS') ? PHPWCMS_ALIAS : ( $aktion[1] == 0 ? 'id='.$aktion[0] : 'aid='.$aktion[1] );
		}
		
		$this->setAlias( $this->alias );
	
		$this->image_src = PHPWCMS_URL.'img/cmsimage.php'.$this->query_separator;
	
	}
	
	function setQuerySeparator($query_separator) {
	
		$this->image_src = PHPWCMS_URL.'img/cmsimage.php'.$query_separator;
	
	}
	
	function setAlias($alias='') {
		if($alias !== '') {
			$this->alias = $alias;
		}
		$this->url = PHPWCMS_URL.'index.php'.returnGlobalGET_QueryString('htmlentities', array(), array('gallery'), $this->alias);
	}

	// +kh: --------  + t
	function setAliasT($alias='') {
		if($alias !== '') {
			$this->alias = $alias;
		}
		$this->url = PHPWCMS_URL.'index.php'.returnGlobalGET_QueryString('htmlentities', array(), array('gallery'), $this->alias);
		// cut out all before &  e.g. indenx.php?nixda&gallery=23  -> indenx.php?nixda
		if (strpos ( $this->url, '&' )) {
			$pos = strpos ( $this->url, '&' );
			$this->url = substr($this->url,0,$pos);
		} 
	}

	function setGallerySort($sort='') {
	
		$this->gallery_sort = $sort;
		$this->_getGallerySortString();
	
	}
// ---------------------------------- +kh
// is flag equal check? 
	function has_flag ($flags, $flag_check) {
		return (($flags & $flag_check) == $flag_check);
	}
	
// ---------------------------------- +kh
// is flag in check?
	function isin_flag ($flags, $flag_check) {
		return (($flags & $flag_check) > 0);
	}
// ----------------------------------


// try to return the matching section of tags
// within tags like [XX] ....[/XX]
// ---------------------------------- +kh
	function my_get_tag_section($s='',$t='') {
	return (preg_match("/\[".$s."\](.*?)\[\/".$s."\]/si", $t, $g)) ? $g[1] : '';
}

	function my_replace_tag_section($s='',$t='',$r='') {
	// try to delete the matching section of own tags
	// within tags like [XX] ....[/XX]
		if (strpos ( $t, ']'."\r\n")) {  // CRLF (0x0D 0x0A)
			$t = preg_replace("/\]\r\n/si", ']', $t);  // replaces all "]CR LF" to "]" behind tags
		}
	return preg_replace("/\[".$s."\](.*?)\[\/".$s."\]/si", $r, $t);  // kill all own tags [XX] and return clean text
}
// ----------------------------------


// special linefeed in lightbox
// lightbox expects e.g.: &lt;br /&gt;  instead of <br />
// Trigger var $line_break = "\n\t"; in caption to <br />
// ---------------------------------- +kh
/**
 * Convert 2 to x line breaks of plain text into correct <p> and <br>
 */
function my_parseLightboxCaption($text='', $encode_function='html_specialchars') {  // like plaintext_htmlencode
	$text = trim($text);
	if($text) {
		if ($this->lightbox_linebreak ) {  // [0|1] [0=off (old behavior) | 1=on (no [BR] needed for linebreak)]
			$text = '[p]' . preg_replace('/\s{0,}\n\s{0,}\n\s{0,}/s', '[/p][p]', $text) . '[/p]';
//			$text =  preg_replace('/\s{0,}\n\s{0,}\n\s{0,}/s', '[/p][p]', $text) ;
			$text = preg_replace('/\s{0,}\n\s{0,}/s', '[br]', $text);
		}
		$text = html_parser($text);
		$text = $encode_function($text);
		$text = str_replace(array('[/p][p]', '[p]', '[/p]', '[br]'), array("</p>\n<p>", '<p>', '</p>', "<br />\n"), $text);
	}
	return $text;
}
	
	
// ----------------------------------

	/**
	 * get images
	 *
	 * Dive into phpwcms's db file storage and get all 
	 * images based on given mode, starting at folder ID.
	 *
	 * @param mixed use integer value for 1 folder or array for multiple 
	 */
	function getImages($folder=0) {
	
		$folders			= array();
		$this->images	= array();

		if( is_array($folder) ) {
			foreach($folder as $item) {
				$item = intval($item);
				$folders[ $item ] = $item;
			}
		} else {
			$folders[] = intval($folder);	
		}
		
		if(count($folders)) {
			$folders = implode(',', $folders);
		}
		
		// check if folder(s) is(are) live
		$sql  = 'SELECT COUNT(*) FROM '.DB_PREPEND.'phpwcms_file WHERE ';
		$sql .= 'f_id IN (' . $folders . ') AND f_kid=0 AND f_trash=0 AND f_aktiv=1 AND f_public=1';
		if(_dbCount($sql) > 0) {
			// catch the images: ($image_limit/$image_sort is set in calling function temp.!!)
			$this->image_limit	= intval($this->image_limit);
			$this->image_sort	= strtoupper($this->image_sort);

// ================= since r304 -> f_sort added -> (http://code.google.com/p/phpwcms/source/detail?r=304)
// string defines the sorting mode
// ASC/DESC = get images sorted by date,    RAND = get images by random)
// +kh NAME-ASC, NAME-DESC
// SORT-ASC, SORT-DESC, SORT-NAME-ASC, SORT-NAME-DESC, SORT-CREATE-ASC, SORT-CREATE-DESC  (since r304)
// NAME-ASC, NAME-DESC, CREATE-ASC, CREATE-DESC
			switch($this->image_sort) {
				case 'ASC':								$order_by = ' ORDER BY f_created ASC';								break;
				case 'RAND':							$order_by = ' ORDER BY RAND()';												break;
				case 'NAME-ASC':					$order_by = ' ORDER BY f_name ASC';										break;  // +kh
				case 'NAME-DESC':					$order_by = ' ORDER BY f_name DESC';									break;  // +kh
				case 'SORT-ASC':					$order_by = ' ORDER BY f_sort ASC';										break;
				case 'SORT-DESC':					$order_by = ' ORDER BY f_sort DESC';									break;
				case 'SORT-NAME-ASC':			$order_by = ' ORDER BY f_sort ASC, f_name ASC';				break;
				case 'SORT-NAME-DESC':		$order_by = ' ORDER BY f_sort DESC, f_name DESC';			break;
				case 'SORT-CREATE-ASC':		$order_by = ' ORDER BY f_sort ASC, f_created ASC';		break;
				case 'SORT-CREATE-DESC':	$order_by = ' ORDER BY f_sort DESC, f_created DESC';	break;
				// special image -> LIST-SPECIAL-IMG 
				case 'LIST-SPECIAL-IMG':				$order_by = ' ORDER BY f_sort ASC, f_changed DESC';				break;
				default:									$order_by = ' ORDER BY f_created DESC';
			}
// =================
			if ($this->image_sort	== 'LIST-SPECIAL-IMG') {  // special image sort set?
				$sql  = 'SELECT * FROM '.DB_PREPEND.'phpwcms_file WHERE ';
				$sql .= 'f_pid IN (' . $folders . ') AND f_kid=1 AND ';
				$sql .= 'f_trash=0 AND f_aktiv=1 AND f_public=1 AND ';
				$sql .= "f_ext IN ('jpg', 'jpeg', 'gif', 'png')";
				$sql .= $order_by;
				if($this->image_limit > 0) { 
					$sql .= ' LIMIT ' . $this->image_limit;
				}
			}
			else {  // regular ============= for simple list gallery and sub sub gallery thumb(s) 
				$sql  = 'SELECT * FROM '.DB_PREPEND.'phpwcms_file WHERE ';
				$sql .= 'f_pid IN (' . $folders . ') AND f_kid=1 AND ';
				$sql .= 'f_trash=0 AND f_aktiv=1 AND f_public=1 AND ';
				$sql .= "f_ext IN ('jpg', 'jpeg', 'gif', 'png')".' AND ';
				$sql .= "f_sort > '-1'";
				$sql .= $order_by;
				if($this->image_limit > 0) { 
					$sql .= ' LIMIT ' . $this->image_limit;
				}
			}
// =================
			
			$result = _dbQuery($sql);
			if(isset($result[0])) {
				$this->images = $result;  // Back referenc of filled images array
				// count the images in selected sub/sub sub gallery and set it to min -height/width
				if (isset ($result[$this->thumb_count_to_min])) {
					$this->thumb_width		= $this->thumb_width_min;
					$this->thumb_height		= $this->thumb_height_min;
				}
			}
		}
	}
	

	/**
	 * get galleries
	 *
	 * Starting with given folder ID an array with all folder
	 * information inside are returned.
	 *
	 */
	function getGalleries($folder_id = 0) {
	
		$folder_id			= intval($folder_id);

		/* yet unsupported 	
		$this->depth		= intval($this->depth);

		if(strtoupper($this->gallery_mode) === 'THREAD' && $this->depth > 0) {
			$this->gallery_mode	= 'THREAD';
		} else {
			$this->gallery_mode	= 'FLAT';
			$this->depth		= 0;
		}
		*/
		
		$sql  = 'SELECT * FROM '.DB_PREPEND.'phpwcms_file WHERE ';
		$sql .= 'f_pid='.$folder_id.' AND f_kid=0 AND f_trash=0 AND f_aktiv=1 AND f_public=1';
		$sql .= $this->gallery_sort;
	
		$this->galleries = _dbQuery($sql);
		$this->proofGallery();
	}


	/**
	 * check gallery array and set names
	 */
	 function proofGallery() {
		if(!isset($this->galleries[0])) {
			$this->galleries = array();
			return;
		}
		if($this->use_name_date === true) {
			foreach($this->galleries as $key => $row) {
				$this->_stripDateFromName( $this->galleries[$key] ); 
			}
		}
	}


	/**
	 * search for gallery name date and strip it.
	 * if some date is found set gallery create date to it
	 */	
	function _stripDateFromName(&$gallery) {
	
		// search gallery name for matching date YYYY-MM-DD
		if( preg_match('/^([0-9]{2}|[0-9]{4})\-([0]{0,1}[0-9])\-([0-9]{1,2})(.+)/', $gallery['f_name'], $match) ) {

			$name				= trim($match[4]);
			$gallery['f_name']	= $name;
			
			$year	= intval($match[1]);
			$month	= intval($match[2]);
			$day	= intval($match[3]);
			
			if($month > 12 || $month == 0) {
				$month = 1;
			}			
			if($day > 31 || $day == 0) {
				$day = 1;
			}

			$date = mktime(11 , 59, 59, $month, $day, $year);
			if($date) {
				$gallery['f_created'] = $date;
			}
		}
	}

	function getGalleriesFolderId($folder=NULL) {
	
		if( $folder !== NULL ) {
			$this->getGalleries( intval($folder) );
		}
		
		$folderId = array();
		
		foreach($this->galleries as $item) {
		
			$folderId[] = $item['f_id'];
		}
		
		return $folderId;
	
	}


	/**
	 * get gallery information
	 */	
	function getGallery($folder_id=0) {
		
		$folder_id		= intval($folder_id);
		
		$sql  = 'SELECT * FROM '.DB_PREPEND.'phpwcms_file WHERE ';
		$sql .= 'f_id='.$folder_id.' AND f_kid=0 AND f_trash=0 AND f_aktiv=1 AND f_public=1';
		$sql .= $this->gallery_sort;
	
		$this->galleries = _dbQuery($sql);
		$this->proofGallery();
		
		if( count($this->galleries) AND isset($this->galleries[0]['f_id']) ) {
		
			$this->gallery = $this->galleries[0];
		
		} else {
		
			$this->gallery = array();
		
		}
	}

	
// =========== since r304 f_sort added
// ASC/DESC = get images sorted by date,    RAND = get images by random)
// +kh NAME-ASC, NAME-DESC
// SORT-ASC, SORT-DESC, SORT-NAME-ASC, SORT-NAME-DESC, SORT-CREATE-ASC, SORT-CREATE-DESC  (since r304)
// NAME-ASC, NAME-DESC, CREATE-ASC, CREATE-DESC
	function _getGallerySortString() {
		
		if( $this->_sort === '' || !is_string( $this->_sort ) ) {
		
			$this->gallery_sort	= strtoupper($this->gallery_sort);
			
			switch($this->gallery_sort) {
			
				case 'NAME-DESC': 				$this->gallery_sort = ' ORDER BY f_name DESC'; 								break;
				case 'SORT-DESC': 				$this->gallery_sort = ' ORDER BY f_sort DESC'; 								break;
				case 'SORT-ASC': 					$this->gallery_sort = ' ORDER BY f_sort ASC'; 								break;
				case 'SORT-NAME-DESC': 		$this->gallery_sort = ' ORDER BY f_sort DESC, f_name DESC'; 	break;
				case 'SORT-NAME-ASC': 		$this->gallery_sort = ' ORDER BY f_sort ASC, f_name ASC'; 		break;
				case 'SORT-CREATE-DESC': 	$this->gallery_sort = ' ORDER BY f_sort DESC, f_created DESC'; break;
				case 'SORT-CREATE-ASC': 	$this->gallery_sort = ' ORDER BY f_sort ASC, f_created ASC'; 	break;
				case 'CREATE-ASC': 				$this->gallery_sort = ' ORDER BY f_created ASC'; 							break;
				case 'CREATE-DESC':				$this->gallery_sort = ' ORDER BY f_created DESC'; 						break;
				default: 									$this->gallery_sort = ' ORDER BY f_name ASC'; // NAME-ASC 
			}
		}
	}

// =========== 

	/**
	 * listGalleries
	 * Output: List of sub galleries
	 */
	function listGalleries($folder=NULL) {
	
		if( $folder !== NULL) {
			$this->getGalleries( intval($folder) );
		}
		
		$listing = array();
		
		// loop galleries
		foreach($this->galleries as $item) {
			
			$listing[] = $this->parse($item);
			
		}
		
		if(count($listing)) {

			// add gallery listing header, see template definitions
			array_unshift($listing, $this->list_header);
		
			// add gallery listing footer, see template definitions
			array_push($listing, $this->list_footer);

			return implode($this->line_break, $listing);
		}
		
		// return empty string
		return '';
	}
	
	/**
	* Detail list with title text /on|off and preview sub thumbs 
	*/
	function showGallery($folder=NULL) {  
	
		if( $folder !== NULL ) {
			$this->getGallery( $folder );
		}
		
		if( count($this->gallery) ) {

			$gallery	= array();
			
			$gallery[]	= $this->detail_header;
			
			// -kh	$gallery[]	= $this->detail_title_prefix . html_specialchars($this->gallery['f_name']) . $this->detail_title_suffix;
			
			// =========== +kh: Title
			// +kh	[0|1|2|3] [off | title=on | date=on | title and date = on] 
			switch ($this->detail_title_date_on) {
			case 0: // no title and/or date
				break;
			case 1: // date is eleminated in galleryx.php
				$gallery[]	= $this->detail_title_prefix . html_specialchars($this->gallery['f_name']) . $this->detail_title_suffix;
				break;
			case 2: // only date
				$gallery[]	= $this->detail_title_prefix . $this->detail_title_suffix;
				break;
			case 3: // title and date 
				$gallery[]	= $this->detail_title_prefix . html_specialchars($this->gallery['f_name']) . $this->detail_title_suffix;
				break;
			}
			// =========== +kh: Show list info in detail output
			if( (trim( $this->gallery['f_longinfo'] ) != '') AND ($this->detail_descr_info) ) {  
				$gallery[]	= $this->detail_descr_prefix . ( plaintext_htmlencode($this->gallery['f_longinfo']) ) . $this->detail_descr_suffix;
			}
			
			$gallery[]	= $this->listGalleryImages( $this->gallery['f_id'] );
			
			// replaces gallery linking
			$this->detail_gallery_back	= str_replace('[/LINK]', '</a>', $this->detail_gallery_back);
			$this->detail_gallery_back	= str_replace('[LINK]', '<a href="' . $this->url . '">', $this->detail_gallery_back);
			// ---------- +kh: if sub_sub gallery only is active -> no gallery_back
			if ($this->gallery_sub_subid == 0) {$gallery[]	= $this->detail_gallery_back; }
			
			$gallery[]	= $this->detail_footer;
			
			$gallery	= implode($this->line_break, $gallery);
			
			$gallery	= render_cnt_date($gallery, $this->gallery['f_created']);

			return $this->parseGeneral( $gallery );
		
		}
		
		return '';
	}
	
	// sub sub gallery output with detail header /on|off, all prev. thumbs and caption /on|off and zoom/lightbox /on|off
	function listGalleryImages($folder, $sort=NULL) { 

		if($sort !== NULL) {
			$temp_sort	= $this->image_sort;
			$this->image_sort	= $sort;
		}
		
		$this->getImages( $folder );

		if($sort !== NULL) {
			$this->image_sort	= $temp_sort;
		}

		/* ---------------------------
		What we need for lightbox e.g.:
					<div class="thumbnail">
						<a href="http://example.tld/img/cmsimage.php/800x600/exyz...123.gif" 
						rel="lightbox[gallery32]" title=" &lt;br /&gt;&lt;br /&gt;Info-Text in &lt;br /&gt;three lines&lt;br /&gt;
						 copyright text">
						<img src="http://example.tld/img/cmsimage.php/120x100/exyz...123.gif" alt="picture" title="Thumb-title " border="0"></a>
					</div>
		------------------------------ */

		$images = array();

		foreach($this->images as $image) {
		
			// -------------- +kh add copyright 
			// Priority: 1. db text, ((2. RT text, 3. file text)  -> figure out in galleryx)
			if ($image['f_copyright'] != '') { $temp_detail_copyr_text = $image['f_copyright']; }
			else { $temp_detail_copyr_text = $this->detail_copyr_text; }
			// ---------------
			
			// =============== +kh parses the caption text for [TT] Title-Text and [AT] Alt-Text
				$my_caption_text 	= '';
				$my_title_tag 		= '';
				$my_alt_tag   		= '';
				$my_caption_title = '';
			
			if(($image['f_longinfo'] != '') ) { // parse for tags in all situations
				$my_title_tag 		= $this->my_get_tag_section('TT', $image['f_longinfo']);
				$my_alt_tag 			= $this->my_get_tag_section('AT', $image['f_longinfo']);
				$my_caption_text 	= $this->my_replace_tag_section('TT', $image['f_longinfo']);	// save caption and kill "[XX]text[/XX]"
				$my_caption_text	= $this->my_replace_tag_section('AT', $my_caption_text); 			// save caption and kill "[XX]text[/XX]"
			}
			// ===============
				// if there is no db input filled up with default text
			if (empty($my_title_tag)) 		{$my_title_tag = empty($this->detail_caption_title) ? html_specialchars( $image['f_name'] ): $this->detail_caption_title; }
			if (empty($my_alt_tag)) 			{$my_alt_tag   = empty($this->detail_caption_alt)   ? html_specialchars( $image['f_name'] ): $this->detail_caption_alt; }
			if (empty($my_caption_text)) 	{$my_caption_text = $this->detail_caption_text; }

			// --- Caption text not formatted in title (hover)
			if ($this->detail_capt_thumb_title_on) {
				$my_title_tag = preg_replace('/\[.*?\]/', '', $my_caption_text);
			}
			
// -kh			$image['f_longinfo'] = trim($image['f_longinfo']);

			$img  = '<img src="';
			$img .= $this->image_src . $this->thumb_width . 'x' . $this->thumb_height;
			if($this->detail_thumbnail_crop == 1) {
				$img .= 'x1';
			}
			$img .= '/' . $image['f_hash'] . '.' . $image['f_ext'];
			$img .= '" alt="' . $my_alt_tag . '" border="0" title="'.$my_title_tag.'" />';

			// --- create zoom
			if($this->zoom === true || $this->lightbox === true) {
				
				$a = '<a href="' . $this->image_src . $this->width . 'x' . $this->height;
				
				if($this->detail_zoom_crop == 1) { 
					$a .= 'x1';
				}
				$a .= '/' . $image['f_hash'] . '.' . $image['f_ext'] . '" target="_blank"';
				
			// Lightbox ----------
				if($this->lightbox === true) {
					$a .= ' rel="lightbox[gallery'.$folder.']"';
				}
				$a .= ' title="';
				
				// [0|1|2|3] [0=off | 1=capt |2=light | 3=capt&light ]   in sub sub gallery
				if(($image['f_longinfo'] != '') and ($this->has_flag($this->detail_caption_on,2))) { // lightbox  = on)) 
					$a .= $this->my_parseLightboxCaption($my_caption_text); // kh parseLightboxCaption
				}
				
				// -------------- +kh add copyright for Lightbox
				// [0=off|1=cap|2=light|3=capt&light|4=down|5=down&capt|6=down&light|7=down&light&capt]
				if ($this->has_flag($this->detail_copyr_on,2)) { // lightbox  = on
				$temp_text = '';
				
					// select the right source (db or text)
					if ($image['f_copyright'] != '') { // db copyright set?
						$temp_text = LF . $this->detail_copyr_text_prefix . $temp_detail_copyr_text ; 
					}
					else { $temp_text = LF .  $temp_detail_copyr_text ; } // all other
					
//					$a .= ( plaintext_htmlencode($temp_text) ) ; // +kh parse for Lightbox  <p>...</p>
					$a .= ( parseLightboxCaption($temp_text) ) ; // +kh special parse for Lightbox
				}
				
				// --------------
				
				$a .= '"';
				$a .= '>';
				$img = $a . $img .'</a>';
				
			} // ---- END create zoom


			// detail_thumbnail_caption ------ */
			// [0|1|2|3] [0=off | 1=capt |2=light | 3=capt&light ]   in sub sub gallery
			if ($this->isin_flag($this->detail_caption_on, 1)) {
				$img .= $this->line_break;
				$img .= $this->detail_caption_prefix .  plaintext_htmlencode($my_caption_text, 'html_parser');
				$img .= $this->detail_caption_suffix;
			}  // END detail_thumbnail_caption

				// -------------- +kh add copyright in caption
				// [0=off|1=cap|2=light|3=capt&light|4=down|5=down&capt|6=down&light|7=down&light&capt]
				if ($this->has_flag($this->detail_copyr_on,1)) { 		// is [1 caption=on]
					$temp_text = '';
					
					$temp_text  = $this->line_break;
					$temp_text .= $this->detail_copyr_prefix . LF; 
					
					// a special prefix to db copyright
					if ($image['f_copyright'] != '') {$temp_text .= $this->detail_copyr_text_prefix . $image['f_copyright']; }
					elseif ($this->detail_copyr_text != '') { $temp_text .= plaintext_htmlencode($temp_detail_copyr_text); }
					
					$temp_text .= $this->detail_copyr_suffix ;

					$img .= $temp_text;	
				} // --------------

			
			$img = $this->detail_thumbnail_prefix . $img . $this->detail_thumbnail_suffix;
			
			// prepare item download -------------------------
			if($this->download_on & 1) {		// [0=off|1=download|3=download & direct]
			
				if(!isset($phpwcms)) {
					global $phpwcms;
				}
				
				$IS_NEWS_CP						= true;
				$crow									= array();
				$value								= array();
				$value['cnt_object']	= array();
				
				// -------------- +kh add copyright 
				// [0=off|1=cap|2=light|3=capt&light|4=down|5=down&capt|6=down&light|7=down&light&capt]
				if ($this->has_flag($this->detail_copyr_on,4) ) { 
				
					if ($image['f_copyright'] != '') {
					
						$value['cnt_object']['cnt_files']	= array( 'id' => array($image['f_id']), 'caption' => $this->detail_copyr_text_prefix . $temp_detail_copyr_text );
					}
					elseif ($this->detail_copyr_text != '') {
						$value['cnt_object']['cnt_files']	= array( 'id' => array($image['f_id']), 'caption' => $temp_detail_copyr_text );
					} 
				} 
				
				else { $value['cnt_object']['cnt_files']	= array( 'id' => array($image['f_id']), 'caption' => ' ' ); }

//				$value['files_direct_download']		= $this->download_direct ? 1 : 0;

				// [0=off|1=download|3=download & direct]
				$value['files_direct_download']		= $this->has_flag($this->download_on,3);
				$value['files_template']					= $this->item_download_template == 'default' ? '' : $this->item_download_template;

				$news 														= array('files_result' => '');
				$content													= array();
				$content['file_static_result'][0]	= $image;
				
				// include content part files renderer
				include(PHPWCMS_ROOT.'/include/inc_front/content/cnt7.article.inc.php');
	
				$img = render_cnt_template($img, 'DOWNLOAD', $news['files_result'] );
			
				unset($IS_NEWS_CP);
			
			} else {
				
				$img = render_cnt_template($img, 'DOWNLOAD', '' );
			
			}
			
			$images[] = $img;

		}
		
		if($this->lightbox === true) {
		
			initializeLightbox();
		
		}

		return implode($this->line_break, $images);
	}

	// --------  + t
	function listImages($folder, $limit=NULL, $sort=NULL) {
	
		if($limit !== NULL) {
			$temp_limit	= $this->image_limit;
			$this->image_limit	= $limit;
		}

		if($sort !== NULL) {
			$temp_sort	= $this->image_sort;
			$this->image_sort	= $sort;
		}

		$this->getImages( $folder );

		if($limit !== NULL) {
			$this->image_limit	= $temp_limit;
		}
		if($sort !== NULL) {
			$this->image_sort	= $temp_sort;
		}

		$images = array();

		foreach($this->images as $image) {
		
			$name = html_specialchars( $image['f_name'] );

			$img  = '<img src="';
			$img .= $this->image_src . $this->list_image_width . 'x' . $this->list_image_height;
			if($this->list_image_crop == 1) {
				$img .= 'x1';
			}
			$img .= '/' . $image['f_hash'] . '.' . $image['f_ext'];
			$img .= '" alt="' . $name . '" border="0" />';
			
			$images[] = $this->list_image_prefix . $img . $this->list_image_suffix;

		}
		
		if(count($images)) {

			// add gallery listing header
			array_unshift($images, $this->list_image_header);
		
			// add gallery listing footer
			array_push($images, $this->list_image_footer);
			
			$images = implode($this->line_break, $images);
			
			// replaces gallery linking
			$images = str_replace('[/LINK]', '</a>', $images);
			$images	= str_replace('[LINK]', '<a href="' . $this->url . '">', $images);

			return $images;
		}
		
		// nothing to return than empty string
		return '';
	}
	
	/**
	* ======= Listing the sub galleries
	*/
	function parse($item, $type='GALLERY_ITEM') {
	
		if($type === 'GALLERY_ITEM') {
		
			$entry	= $this->list_template;
			
			$name	= html_specialchars( $item['f_name'] );
			$link	= $this->url . '&amp;gallery=' . $item['f_id'];
			// -kh	$title	= $this->list_title_prefix . $name . $this->list_title_suffix;
			
			// +kh [0|1|2|3] [off | title=on | date=on | title and date = on]
			switch ($this->list_title_date) {
			case 0: // no title and/or date
				if ($this->list_thumbnail > 0) { $title	= ''; } // only if there is a thumb set 
				break;
			case 1: // only title, date is eleminated in galleryx.php
				$title	= $this->list_title_prefix . $name . $this->list_title_suffix;
				break;
			case 2: // only date  (date is organized in galleryx.php includet in list_title_prefix)
				$title	= $this->list_title_prefix . $this->list_title_suffix;
				break;
			case 3: // title and date 
				$title	= $this->list_title_prefix . $name . $this->list_title_suffix;
				break;
			default:
				$title	= $this->list_title_prefix . $name . $this->list_title_suffix;
				break;
			}
			
			// [0|1|2|3] [0=off | 1=info |2=off | 3=info and date ]   in sub gallery (list)
			if ($this->isin_flag($this->list_descr_info_date_on, 1)) {
				$descr	= trim( $item['f_longinfo'] );
			} else { $descr = '';}
			
			if($descr !== '') {
				$descr = $this->list_descr_prefix . ( plaintext_htmlencode( $descr ) ) . $this->list_descr_suffix;
			}
			
			
			$thumbs = array();
			
			// get preview/thumbnail image(s) for this gallery
			if($this->list_thumbnail > 0) {
				$temp_limit	= $this->image_limit;  // Save sub sub parameters to temp.
				$temp_sort	= $this->image_sort;
				
				$this->image_limit	= $this->list_thumbnail;			// Number of images
				$this->image_sort		= $this->list_thumbnail_sort; // Sort option for images in List gallery
				$this->getImages( $item['f_id'] );								// catching images
				
				foreach($this->images as $image) {
				
					$img  = '<img src="';
					$img .= $this->image_src . $this->list_thumbnail_width . 'x' . $this->list_thumbnail_height;
					if($this->list_thumbnail_crop == 1) {
						$img .= 'x1';
					}
					$img .= '/' . $image['f_hash'] . '.' . $image['f_ext'];
					$img .= '" alt="' . $name . '" border="0" />';
					
					$thumbs[] = $this->list_thumbnail_prefix . $img . $this->list_thumbnail_suffix;
			
				}
				
				$this->image_limit	= $temp_limit;
				$this->image_sort		= $temp_sort;
			
			}

			$thumbs	= implode(LF, $thumbs);
			
			$entry	= str_replace('{TITLE}', $title, $entry);
			if ($this->list_thumbnail_caption ) { 
				$entry	= str_replace('{DESCRIPTION}', $descr, $entry);
			} else { $entry	= str_replace('{DESCRIPTION}', '', $entry); }
			$entry	= str_replace('{THUMBNAIL}', $thumbs, $entry);
			$entry	= str_replace('[/LINK]', '</a>', $entry);
			$entry	= str_replace('[LINK]', '<a href="' . $link . '" title="' . $name . '">', $entry);
			$entry	= render_cnt_date($entry, $item['f_created']);
			
			return $this->parseGeneral( $entry );
		}
	
	}
	
	function parseGeneral($string='') {
	
		$string = str_replace( '[LF]', LF, $string);
		$string = str_replace( '[TAB]', "\t", $string);
	
		return $string;
	}
	

	function getGalleryTree() {
	
		$sql  = 'SELECT * FROM '.DB_PREPEND.'phpwcms_file ';
		$sql .= 'WHERE f_kid=0 AND f_aktiv=1 AND f_public=1';
		if($this->gallery_only) {
			$sql .= ' AND f_gallerystatus=2';
		}
		$this->gallery_rootid = intval($this->gallery_rootid);
		if($this->gallery_rootid > 0) {
			$sql .= ' AND f_pid='.$this->gallery_rootid;
		}
		$sql .= $this->gallery_sort;
	
		return _dbQuery($sql);
	
	}
	
	function getGallerySub($parent=0) {

		$sql  = 'SELECT * FROM '.DB_PREPEND.'phpwcms_file ';
		$sql .= 'WHERE f_kid=0 AND f_aktiv=1 AND f_public=1';
		if($this->gallery_only) {
			$sql .= ' AND f_gallerystatus IN(3,2)';
		}
		$sql .= ' AND f_pid='.intval($parent);
		$sql .= $this->gallery_sort;
	
		return _dbQuery($sql);
	
	}
	
}

?>